package at.fhooe.mc.magicwand.client.controller;

import at.fhooe.mc.magicwand.client.request.*;
import at.fhooe.mc.magicwand.client.exceptions.CompassException;
import at.fhooe.mc.magicwand.client.exceptions.GPSException;
import at.fhooe.mc.magicwand.client.exceptions.ParsingException;
import at.fhooe.mc.magicwand.client.exceptions.ServerConnectionException;
import at.fhooe.mc.magicwand.client.network.*;
import at.fhooe.mc.magicwand.client.gui.GuiMIDlet;
import at.fhooe.mc.magicwand.client.poi.POI;
import at.fhooe.mc.magicwand.client.poi.XMLParser;
import java.util.Vector;

/**
 * @class ControlClass
 * @author Maria Neumayer
 * @author Manuela Vogler
 * @brief Interface between GUI and other parts of the project
 */
public class ControlClass implements Runnable {

    /**
     * The instance of the RequestCreator class which is used to create a new
     * request
     */
    private RequestCreator m_creator;
    /**
     * The connection to the server
     */
    private ClientServerConnection m_connection;
    /**
     * if the server response was successful
     */
    private boolean m_successfulResponse;
    /**
     * Array in which the request options (e.g. maximum distance) are stored.
     * If no options were set, the array is null.
     */
    private String[] m_options;
    /**
     * Parses received xml data to a POI array
     */
    private XMLParser m_parser;
    /**
     * Vector with received POI
     */
    private Vector m_poi;
    /**
     * Makes an array with all available categories in the POI vector
     */
    private String[] m_categories;
    /**
     * POI Vector with filters and options followed
     */
    private Vector m_poiOptions;
    /**
     * server address of socket
     */
    private String m_address;
    /**
     * port of socket
     */
    private int m_port;
    /**
     * Defines if the Client-Server-Connection is opened
     */
    private boolean m_connectionOpened;
    /**
     * Defines if the Compass-Connection is opened
     */
    private boolean m_compassConnOpened;
    /**
     * Defines if the thread is running
     */
    private boolean m_threadFlag;
    /**
     * Defines if a Request is made at the moment
     */
    private boolean m_makeRequest;
    /**
     * Defines if a connection is openend at the moment
     */
    private boolean m_openConnection;
    /**
     * GUIMIDlet to set result- or main-Screen
     */
    private GuiMIDlet m_gui;
    /**
     * Exception of request
     */
    private Exception m_exception;

    /**
     * @fn public ControlClass()
     * @brief Constructs a new ControlClass, connections to the server and to the
     * compass are not established.
     */
    public ControlClass() {
        m_connectionOpened = false;
        m_compassConnOpened = false;
        m_successfulResponse = false;
        m_makeRequest = false;
        m_threadFlag = true;
        m_openConnection = false;
    }

    /**
     *
     * @fn public void openConnection(String _address, int _port)
     * @brief Starts the thread that opens the connection
     * @param _address server address
     * @param _port port of socket
     */
    public void openConnection(String _address, int _port) {
        m_address = _address;
        m_port = _port;
        m_openConnection = true;
    }

    /**
     *
     * @fn public void openingConnection() throws ServerConnectionException, CompassException
     * @brief Opens connections to server and compass and starts the XMLParser.
     * @throws ServerConnectionException
     *             if connection to server couldn't be opened
     * @throws CompassException
     *             if connection to compass couldn't be opened
     */
    public void openingConnection() throws ServerConnectionException,
            CompassException {
        try {
            //Connection to server is established
            m_connection = new ClientServerConnection(m_address, m_port);
            m_connectionOpened = true;

            //The RequestCreator is initialized. If no exception occurs, the
            //connection to the compass has been successfully established
            m_creator = new RequestCreator();
            m_compassConnOpened = true;

            //The XMLParser is initialized.
            m_parser = new XMLParser();
        } catch (ServerConnectionException _e) {
            //Connection to server couldn't be established.
            m_connectionOpened = false;
            throw new ServerConnectionException(
                    "Connection to server couldn't be established.");
        } catch (CompassException _e) {
            //Connection to compass couldn't be established.
            m_compassConnOpened = false;
            throw new CompassException(
                    "Connection to compass couldn't be established.");
        }
    }

    /**
     * @fn public void makeRequest()
     * @brief Uses the RequestCreator class to make a new request, send it to
     *        the server and get the response xml.
     * @throws ServerConnectionException
     *             if request can't be sent or response can't be received
     * @throws CompassException
     *             if compass data is not available
     * @throws GPSException
     *             if gps data is not available
     * @throws ParsingException
     *             if received answer is not valid
     */
    public void makeRequest() throws ServerConnectionException,
            CompassException, GPSException, ParsingException {
        m_successfulResponse = false;
        String requestMessage;

        //Creates request message by using the RequestCreator.
        if (m_options != null) {
            //Options are set, uses them to create the message
            requestMessage = m_creator.createRequest(m_options[0],
                    m_options[1], m_options[2]);
        } else {
            //No options set
            requestMessage = m_creator.createRequest(null, null, null);
        }

        //Sends request message to the server, receives the response and
        //stores the response message in responseXML
        String responseXml = m_connection.sendAndReceive(requestMessage);

        //Parses the xml data into a Vector of POIs,
        //sets colours for result and makes an array with all categories
        if (responseXml != null) {
            m_parser.parseXML(responseXml);
            m_poi = m_parser.getM_poiVector();
            setColours();
            makeCategoryArray();
        }
        m_successfulResponse = (responseXml != null);
    }

    /**
     * @fn public void start()
     * @brief starts the thread
     */
    public void start() {
        Thread t = new Thread(this);
        t.start();
    }

    /**
     * @fn public void run()
     * @brief run function of the thread to make a request
     */
    public void run() {
        while (m_threadFlag == true) {
            if (m_makeRequest == true) {
                m_exception = null;
                try {
                    makeRequest();
                } catch (ServerConnectionException _e) {
                    m_exception = _e;
                } catch (CompassException _e) {
                    m_exception = _e;
                } catch (GPSException _e) {
                    m_exception = _e;
                } catch (ParsingException _e) {
                    m_exception = _e;
                }
                m_gui.setScreen(m_exception);
                m_makeRequest = false;
            } else if (m_openConnection == true) {
                m_exception = null;
                try {
                    openingConnection();
                } catch (ServerConnectionException _e) {
                    m_exception = _e;
                } catch (CompassException _e) {
                    m_exception = _e;
                }
                m_gui.setMainScreen(m_exception);
                m_openConnection = false;
            }
        }
    }

    /**
     * @fn public void setColours()
     * @brief sets the priority colour for POI
     */
    public void setColours() {
        int max;
        if (m_options == null) {
            max = 1000;
        } else {
            max = Integer.parseInt(m_options[0]);
        }
        for (int i = 0; i < m_poi.size(); i++) {
            if (((POI) m_poi.elementAt(i)).getM_distance() < (max / 3)) {
                ((POI) m_poi.elementAt(i)).setM_group(1);
            } else if (((POI) m_poi.elementAt(i)).getM_distance() < (max / 3 * 2)) {
                ((POI) m_poi.elementAt(i)).setM_group(2);
            } else {
                ((POI) m_poi.elementAt(i)).setM_group(3);
            }
        }
    }

    /**
     * @fn public void makeVector(String[] _categories, String _filter)
     * @param _categories
     *            array with categories, specified of the user, is null if not
     *            set
     * @param _filter
     *            which data should be shown, is null if not set
     * @brief makes a vector with relevant data for the GUI
     */
    public void makeVector(String[] _categories, String _filter) {

        Vector poi = new Vector(m_poi.size());
        for (int i = 0; i < m_poi.size(); i++) {
            poi.addElement(((POI) m_poi.elementAt(i)).clone());

        }
        // no options set
        if (_categories == null && _filter == null) {
            for (int i = 0; i < poi.size(); i++) {
                POI current = (POI) poi.elementAt(i);
                current.setM_description(null);
                current.setM_link(null);
                poi.setElementAt(current, i);
            }
        // only filter options set
        } else if (_categories == null) {
            for (int i = 0; i < poi.size(); i++) {
                POI current = (POI) poi.elementAt(i);
                if (_filter.equals("Name")) {
                    current.setM_description(null);
                    current.setM_link(null);
                } else if (_filter.equals("Name, description")) {
                    current.setM_link(null);
                }
                poi.setElementAt(current, i);
            }
        // only category options set
        } else if (_filter == null) {
            int size = poi.size();
            int i = 0;
            while (i < size) {
                POI current = (POI) poi.elementAt(i);
                boolean found = false;
                for (int j = 0; j < _categories.length; j++) {
                    if (_categories[j] != null && _categories[j].equals(current.getM_category())) {
                        found = true;
                    }
                }
                if (found == false) {
                    poi.removeElementAt(i);
                    size--;
                } else {
                    i++;
                }
            }
        // both options set
        } else {
            int size = poi.size();
            int i = 0;
            while (i < size) {
                POI current = (POI) poi.elementAt(i);
                boolean found = false;
                for (int j = 0; j < _categories.length; j++) {
                    if (_categories[j] != null && _categories[j].equals(current.getM_category())) {
                        found = true;
                    }
                }
                if (found == false) {
                    poi.removeElementAt(i);
                    size--;
                } else {
                    if (_filter.equals("Name")) {
                        current.setM_description(null);
                        current.setM_link(null);
                    } else if (_filter.equals("Name, description")) {
                        current.setM_link(null);
                    }
                    poi.setElementAt(current, i);
                    i++;
                }
            }
        }
        m_poiOptions = poi;
    }

    /**
     * @fn public void makeCategoryArray()
     * @brief makes an array with all available categories in the vector
     */
    public void makeCategoryArray() {
        String[] c = new String[m_poi.size()];
        int count = 0;
        boolean noCategory = false;
        for (int i = 0; i < m_poi.size(); i++) {
            POI current = (POI) m_poi.elementAt(i);
            if (current.getM_category() != null) {
                boolean found = false;
                for (int j = 0; j < count; j++) {
                    if (c[j].equals(current.getM_category())) {
                        found = true;
                    }
                }
                if (found == false) {
                    c[count++] = current.getM_category();
                }
            } else {
                noCategory = true;
            }
        }
        if (noCategory == true) {
            c[count++] = "No category";
        }
        String[] result = new String[count];
        for (int i = 0; i < count; i++) {
            result[i] = c[i];
        }
        m_categories = result;
    }

    /**
     * @fn public String[] getM_categories()
     * @brief gets an array with available categories
     * @return array with available categories
     */
    public String[] getM_categories() {
        return m_categories;
    }

    /**
     * @fn public Vector getM_poiOptions()
     * @brief gets the vector with informations about the POI
     * @return poi vector with relevant data for the GUI
     */
    public Vector getM_poiOptions() {
        return m_poiOptions;
    }

    /**
     * @fn public boolean isRequestSuccessful()
     * @brief checks if the request was successful
     * @return if request was successful
     */
    public boolean isRequestSuccessful() {
        return m_successfulResponse;
    }

    /**
     * @fn public void exitConnection() throws ServerConnectionException
     * @brief Closes connection to the server.
     * @throws ServerConnectionException
     *             if server connection couldn't be closed
     */
    public void exitConnection() throws ServerConnectionException {
        try {
            //closes connection to the server
            m_connection.closeConnection();
            m_connectionOpened = false;
        } catch (ServerConnectionException _e) {
            m_connectionOpened = true;
            throw new ServerConnectionException(
                    "Connection to server couldn't be closed");
        }
    }

    /**
     * @fn public void exitCompass() throws CompassException
     * @brief Closes the connection to the compass.
     * @throws CompassException
     *             if compass connection couldn't be closed
     */
    public void exitCompass() throws CompassException {
        try {
            //closes connection to the compass
            m_creator.closeCompassSocket();
            m_compassConnOpened = false;
        } catch (CompassException _e) {
            m_compassConnOpened = true;
            throw new CompassException(
                    "Compass connection couldn't be closed");
        }
    }

    /**
     * @fn public void setOptions(String[] _options)
     * @brief to set specific options of a search
     * @param _options
     *            distance, accuracy and max. results
     */
    public void setOptions(String[] _options) {
        m_options = _options;
    }

    /**
     * @fn public boolean isM_connectionOpened()
     * @brief checks if the ClientServerConnection is opened
     * @return if server connection is open
     */
    public boolean isM_connectionOpened() {
        return m_connectionOpened;
    }

    /**
     * @fn public boolean isM_compassConnOpened()
     * @brief checks if the compass connection is opened
     * @return if compass connection is open
     */
    public boolean isM_compassConnOpened() {
        return m_compassConnOpened;
    }

    /**
     * @fn public boolean isM_threadFlag()
     * @brief if thread is running
     * @return status of thread
     */
    public boolean isM_threadFlag() {
        return m_threadFlag;
    }

    /**
     * @fn public void setM_threadFlag(boolean _flag)
     * @brief sets thread active or inactive
     * @param _flag status of thread
     */
    public void setM_threadFlag(boolean _flag) {
        m_threadFlag = _flag;
    }

    /**
     * @fn public void setM_makeRequest(boolean _request
     * @brief sets m_makeRequest to start or stop the request
     * @param _request sets m_makeRequest
     */
    public void setM_makeRequest(boolean _request) {
        m_makeRequest = _request;
        m_connection.setM_receive(_request);
        m_creator.set_receive(_request);
    }

    /**
     * @fn public void setM_gui(GuiMIDlet _gui)
     * @brief sets the GuiMIDlet for setting main- or resultScreen
     * @param _gui GuiMIDlet
     */
    public void setM_gui(GuiMIDlet _gui) {
        m_gui = _gui;
    }
}
