package at.fhooe.mc.magicwand.client.network;

import at.fhooe.mc.magicwand.client.exceptions.ServerConnectionException;

/**
 * @class MwProtocol
 * @brief Contains all the information used to create valid messages and to communicate with the server.
 * Is also able to check if received messages are valid.
 * @author Manuela Vogler
 */
public class MwProtocol {

    /**
     * Preambel sequence
     */
    private String m_preambel = "MAGIC_WAND_PROTOCOL:";

    /**
     * Sequence between protocol information and data
     */
    private String m_sepchar = "#";

    /**
     * Closing message
     */
    private String m_closeMsg = "byebye";

    /**
     * Character indicating the end of a server response
     */
    private char m_messageEndChar = '\0';

    /**
     * Length of response data according to protocol information
     */
    private int m_curlength;

    /**
     * Server's response data in XML format
     */
    private String m_xmlAnswer;

    /**
     * @fn public String getXmlAnswer()
     * @brief Returns the server's response data in XML format.
     * @return The response data
     */
    public String getXmlAnswer() { 
        return m_xmlAnswer;
    }

    /**
     * @fn public char getMessageEndChar()
     * @brief Returns the character indicating the end of a server message.
     * @return The end character
     */
    public char getMessageEndChar() {
        return m_messageEndChar;
    }

    /**
     * @fn public String getPreambel()
     * @brief Returns the sequence a valid message has to start with.
     * @return The preamble sequence
     */
    public String getPreambel() {
        return m_preambel;
    }

    /**
     * @fn public String getSepchar()
     * @brief Returns the sequence between protocol information and data.
     * @return The seperating sequence
     */
    public String getSepchar() {
        return m_sepchar;
    }

    /**
     * @fn public String getCloseMessage()
     * @brief Returns the message which has to sent to the server before closing the connection.
     * @return The closing message
     */
    public String getCloseMessage() {
        return m_closeMsg;
    }

    /**
     * @fn public MwProtocol()
     * @brief Constructs a new MagicWand protocol.
     */
    public MwProtocol() {
        m_curlength = -1;
    }

    /**
     * @fn public void processData(String _message) throws ServerConnectionException
     * @brief Checks if a received message is valid and stores the response data.
     * The message has to start with the preambel, contain the sepchar and the checksum
     * has to be the same as the message's length.
     * @param _message The received message
     * @throws ServerConnectionException if message doesn't conform to protocol
     */
    public void processData(String _message) throws ServerConnectionException {
        if (!_message.startsWith(m_preambel)) { //message not valid
            throw new ServerConnectionException();
        }
        m_xmlAnswer = ""; //reset answer
        _message = _message.substring(m_preambel.length()); //cut off preambel
        int index = _message.indexOf(m_sepchar);
        m_curlength = Integer.parseInt(_message.substring(0, index)); //get value of checksum
        _message = _message.substring(index + m_sepchar.length()); //get response information
        if (_message.length() == m_curlength) { //message complete
            m_xmlAnswer = _message;
            m_curlength = -1; //reset length
        } else { //checksum and message's length not equal
            throw new ServerConnectionException();
        }
    }
}
