/*
 ============================================================================
 Name		: CompassServer.cpp
 Author	  : Manuela Vogler
 Version	 : 1.0
 Copyright   : 
 Description : CCompassServer implementation
 ============================================================================
 */
#include "CompassServer.h"
// protocol literals
_LIT8(KClientRequest,"get");
_LIT8(KClientClose,"byebye");
_LIT8(KAck,"ok");

//gui output literals
_LIT(KStarted,"Server gestartet,\nwartet auf Client");
_LIT(KStopped,"Server angehalten,\nfortsetzen mit\nOptionen --> Start");
_LIT(KConnError,"Verbindungsabbruch,\nfortsetzen mit\nOptionen->Start");
_LIT(KConnected,"Verbunden mit Client");
_LIT(KRequest,"Anfrage erhalten");
_LIT(KSend,"Anfrage erfolgreich\nbeantwortet");
_LIT(KReaderErr,"Kompass konnte nicht\ngestartet werden");
_LIT(KConnectingErr,"Fehler beim\nVerbindungsaufbau");
_LIT(KProcessErr,"Fehler beim Bearbeiten\nder Anfrage");
_LIT(KClosingErr,"Fehler beim\nVerbindungsabbruch");
_LIT(KErr,"Fehler");


/**
 * @brief Constructor for performing 1st stage construction.
 * Constructs the CCompassServer instance.
 */
CCompassServer::CCompassServer() :
	CActive(EPriorityStandard), iPort(8100) // set standard priority and port
	{
	iAddress.SetAddress(INET_ADDR(127,0,0,1)); //set address to localhost
	CActiveScheduler::Add(this); 
	}

/**
 * @brief Two-phased constructor.
 * Constructed instance is left on the cleanup stack.
 * @return New instance of CCompassServer
 */
CCompassServer* CCompassServer::NewLC()
	{
	CCompassServer* self = new (ELeave) CCompassServer();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

/**
 * @brief Two-phased constructor.
 * Constructed instance is not left on the cleanup stack.
 * @return New instance of CCompassServer
 */
CCompassServer* CCompassServer::NewL()
	{
	CCompassServer* self = CCompassServer::NewLC();
	CleanupStack::Pop(self);
	return self;
	}

/**
 * @brief Constructor for performing 2nd stage construction
 * Part of the construction which can cause a leave.
 */
void CCompassServer::ConstructL()
	{
	User::LeaveIfError(iSocketServer.Connect()); //connect socket server
	}

/**
 * @brief Destructor.
 * Stops the server, closes all resources and destroys instance variables.
 */
CCompassServer::~CCompassServer()
	{
	Stop(); //stop server

	iSocketServer.Close(); //close socket server
	delete iReader;
	delete iMessage;
	}

/**
 * @brief Cancels all the socket's outstanding operations.
 */
void CCompassServer::DoCancel()
	{
	iSocket.CancelAll();
	iActiveSocket->CancelAll();
	}

/**
 * @brief Starts the server and wait for a connection.
 */
void CCompassServer::StartL()
	{
	TInt result(0);

	result
			= iSocket.Open(iSocketServer, KAfInet, KSockStream,
					KProtocolInetTcp);

	if (result != KErrNone) //error while opening socket
		{
		iSocketServer.Close();
		User::Leave(result);
		}

	iAddress.SetPort(iPort);
	iAddress.SetFamily(KAfInet);

	// bind
	User::LeaveIfError(iSocket.Bind(iAddress));

	// listen, maximum number of connections is 1
	User::LeaveIfError(iSocket.Listen(1));

	// close old connection - if any
	iAcceptedSocket.Close();

	// open abstract socket
	User::LeaveIfError(iAcceptedSocket.Open(iSocketServer));

	iActiveSocket = &iAcceptedSocket;

	//accept (asynchronous operation)
	iSocket.Accept(iAcceptedSocket, iStatus);

	iServerState = EConnecting;
	iContainer->SetServerStarted(ETrue);
	iContainer->SetLabelTextL(KStarted);

	// Set the Active Object Active, waiting for the completion of the accept function
	SetActive();

	}

/**
 * @brief Stops the server, closes the socket.
 */
void CCompassServer::Stop()
	{
	Cancel(); //cancel outstanding operations
	iAcceptedSocket.Close();
	iSocket.Close();
	iActiveSocket->Close();
	iContainer->SetServerStarted(EFalse);
	TRAPD(err,iContainer->SetLabelTextL(KStopped));
	}

/**
 * @brief Called after the completion of an asynchronous service. Decides how to proceed depending on the server's status.
 */
void CCompassServer::RunL()
	{
	if ((iStatus == KErrDisconnected) || (iStatus == KErrEof) || (iStatus
			== KErrAbort)) //error occured, asynchronous function not finished successfully
		{
		Stop(); //stop the server
		iContainer->SetLabelTextL(KConnError);
		return;
		}
	else //no error
		{
		switch (iServerState)
			{
			case EConnecting: //client is establishing connection
				RequestData(); //read data from socket
				iServerState = EConnected;
				break;
			case EConnected: //connection successfully established
				RequestData(); //read data from socket
				iContainer->SetLabelTextL(KConnected);
				iServerState = EReceiving;
				break;
			case EReceiving: //receiving request
				RequestData(); //read data from socket
				if (iServerState != EReset) //if request is not to close the socket
					{
					iServerState = ESending;
					iContainer->SetLabelTextL(KRequest);
					}
				break;
			case ESending: //sending response
				RequestData(); //read data from socket
				iContainer->SetLabelTextL(KSend);
				iServerState = EReceiving; //back to waiting for request
				break;
			case EReset: //client wants to close connection
				RequestData(); //read from socket
				Stop(); //stop the server to be able to restart it
				StartL(); //restart, wait for new client connection
			}
		}
	}

/**
 * @brief Reads data from socket and processes it if necessary.
 */
void CCompassServer::RequestData()
	{
	if (iActiveSocket)
		{
		iActiveSocket->RecvOneOrMore(iBuffer, 0, iStatus, iLen); //read data into iBuffer

		if (iServerState == EReceiving || iServerState == EConnecting) //processing of data is necessary
			{
			ProcessData();
			}
		SetActive(); //set active again
		}
	}

/**
 * @brief Processes an incoming request. Checks which request is received and reacts to this request.
 */
void CCompassServer::ProcessData()
	{
	if (iServerState == EConnecting) //establishing connection, no message sent
		{
		TInt err = 0;
		if (!iReader) //start reader if not already running
			{
				TRAP(err,iReader = CCompassReader::NewL());
			}
		if (err != 0)
			{
			TRAPD(err,iContainer->SetLabelTextL(KReaderErr));
			}
		else // reader is now running, send ack to client
			{
			TBuf8<KAnswerSize> answer;
			answer.Append(KAck);
			answer.Append('\0');
				TRAP(err,iMessage = HBufC8::NewL(answer.Length()));
			if (err == 0)
				{
				iMessage->Des().Copy(answer); //copy answer message to iMessage
				// stop socket
				if (iActiveSocket)
					{
					iActiveSocket->CancelRead();
					iActiveSocket->CancelSend();
					}
				if (IsActive()) //if active object has outstanding request
					{
					Cancel(); //cancel the request
					}
				iActiveSocket->Send(*iMessage, 0, iStatus); //send the ack
				User::WaitForAnyRequest();
				}
			else
				{
				TRAPD(err,iContainer->SetLabelTextL(KConnectingErr));
				}
			}
		}
	else if (iServerState == EReceiving) //receiving request
		{
		TPtrC8 msg = iBuffer.Mid(2); //cut off first 2 characters (always received, but don't belong to client's message)
		if (msg.Compare(KClientRequest) == 0) //check if client wants to get compass angle
			{
			TBuf8<KAnswerSize> answer;
			if (iReader->IsRunning()) //reader running
				{
				TInt angle = iReader->GetAngle();
				answer.AppendNum(angle);
				answer.Append('\0');
				}
			else //reader not running
				{
				answer.AppendNum(-1); //signalizes that reader is not running
				answer.Append('\0');
				}
			TRAPD(err,iMessage = HBufC8::NewL(answer.Length()));
			if (err == 0)
				{
				iMessage->Des().Copy(answer); //copy answer message to iMessage
				// stop socket
				if (iActiveSocket)
					{
					iActiveSocket->CancelRead();
					iActiveSocket->CancelSend();
					}
				if (IsActive()) //if active object has outstanding request
					{
					Cancel(); //cancel request
					}
				iActiveSocket->Send(*iMessage, 0, iStatus); //send response with angle
				User::WaitForAnyRequest();
				}
			else
				{
				TRAPD(err,iContainer->SetLabelTextL(KProcessErr));
				}
			}
		else if (msg.Compare(KClientClose) == 0) //check if client wants to close connection, send ack
			{
			TBuf8<KAnswerSize> answer;
			answer.Append(KAck);
			answer.Append('\0');
			TRAPD(err,iMessage = HBufC8::NewL(answer.Length()));
			if (err == 0)
				{
				iMessage->Des().Copy(answer); //copy answer message to iMessage
				// stop socket
				if (iActiveSocket)
					{
					iActiveSocket->CancelRead();
					iActiveSocket->CancelSend();
					}
				if (IsActive()) //if active object has outstanding request
					{
					Cancel(); //cancel request
					}
				iActiveSocket->Send(*iMessage, 0, iStatus); //send ack
				User::WaitForAnyRequest();
				iServerState = EReset;
				}
			else
				{
				TRAPD(err,iContainer->SetLabelTextL(KClosingErr));
				}
			}
		}
	}

/**
 * @brief Handles leaves from RunL function.
 * @param aError The error code
 * @return The error code
 */
TInt CCompassServer::RunError(TInt aError)
	{
	TRAPD(err,iContainer->SetLabelTextL(KErr)); //output in GUI
	return aError;
	}

/**
 * @brief Sets a pointer to the GUI container. This reference is needed for the output of status messages.
 * @param aContainer Reference to the GUI container
 */
void CCompassServer::SetContainerReference(
		CMagicWandServerContainer* aContainer)
	{
	iContainer = aContainer;
	}
